@@ -14,38 +14,33 @@ Written by Andreas Jung
 
 E-Mail: andreas@andreas-jung.com
 
-$Id: TextIndexNG.py,v 1.144 2004/04/01 06:26:49 ajung Exp $
+$Id: TextIndexNG.py,v 1.167 2005/05/19 10:22:35 ajung Exp $
 """
 
 import sys
-from types import StringType, UnicodeType 
 
 from Globals import DTMLFile, InitializeClass
 from AccessControl.SecurityInfo import ClassSecurityInfo
-from zLOG import ERROR, WARNING
-from zLOG import LOG as LOG1
+from zLOG import ERROR, WARNING, LOG
 from OFS.SimpleItem import SimpleItem
 from Products.PluginIndexes import PluggableIndex       
 from Products.PluginIndexes.common.util import parseIndexRequest
 from OFS.content_types import guess_content_type
-from BTrees.IIBTree import IISet, difference, union
+from BTrees.IIBTree import IISet, difference 
 from classVerify import verifyClass
 
 from Products.TextIndexNG2.ResultSet import ResultSet
-from Products.TextIndexNG2.Registry import ParserRegistry, ConverterRegistry, NormalizerRegistry, StorageRegistry
+from Products.TextIndexNG2.Registry import ParserRegistry, ConverterRegistry, NormalizerRegistry, StorageRegistry, ThesaurusRegistry
 from Products.TextIndexNG2.Registry import LexiconRegistry, SplitterRegistry, StopwordsRegistry, RegistryException
 from ParseTree import Evaluator
 
 from interfaces.IStopwords import StopwordsInterface
 
-import parsers, converters, normalizers
-import storages, lexicons, splitters, stopwords
+import parsers, normalizers
+import storages, lexicons, splitters, stop_words
 
-import Stemmer 
 import indexsupport
 import PositionMap
-from Timer import Timer
-from logger import LOG
 
 from AccessControl.Permissions import  search_zcatalog
 try:
@@ -109,9 +104,9 @@ class TextIndexNG(SimpleItem):
     )
 
     _all_options = ('splitter_max_len', 'use_splitter', "splitter_separators",
-         'splitter_single_chars', 'splitter_casefolding', 'use_stemmer', 
+         'splitter_single_chars', 'splitter_casefolding', 
          'lexicon', 'near_distance', 'truncate_left', 'autoexpand',
-         'autoexpand_limit', 'numhits',
+         'autoexpand_limit', 'numhits', 'use_storage', 'use_thesaurus', 'thesaurus_mode',
          'use_stopwords', 'use_normalizer', 'use_converters',
          'use_parser', 'indexed_fields', 'default_encoding'
         )
@@ -120,8 +115,15 @@ class TextIndexNG(SimpleItem):
                      'numhits')
 
     def __init__(self, id, extra=None, caller=None):
+
+        def _get(o, k, default):
+            """ return a value for a given key of a dict/record 'o' """
+            if isinstance(o, dict):
+                return o.get(k, default)
+            else:
+                return getattr(o, k, default)
         
-        self.id            = id
+        self.id = id
 
         # check parameters
         if extra:
@@ -135,92 +137,90 @@ class TextIndexNG(SimpleItem):
             self.catalog_path = None
 
         # indexed attributes
-        self._indexed_fields = getattr(extra, 'indexed_fields', '').split(',')
+        self._indexed_fields = _get(extra, 'indexed_fields', '').split(',')
         self._indexed_fields = [ attr.strip() for attr in  self._indexed_fields if attr ]
         if not self._indexed_fields:
             self._indexed_fields = [ self.id ]
 
         # splitter to be used
-        self.use_splitter = getattr(extra, 'use_splitter', 'TXNGSplitter')
+        self.use_splitter = _get(extra, 'use_splitter', 'TXNGSplitter')
 
         # max len of splitted words
-        self.splitter_max_len= getattr(extra,'splitter_max_len', 64)
+        self.splitter_max_len= _get(extra, 'splitter_max_len', 64)
 
         # allow single characters
-        self.splitter_single_chars   = getattr(extra,'splitter_single_chars',0)
+        self.splitter_single_chars = _get(extra,'splitter_single_chars',0)
 
         # valid word separators
-        self.splitter_separators = getattr(extra, 'splitter_separators','.+-_@')
+        self.splitter_separators = _get(extra, 'splitter_separators','.+-_@')
 
         # allow single characters
-        self.splitter_casefolding   = getattr(extra,'splitter_casefolding',1) 
+        self.splitter_casefolding = _get(extra,'splitter_casefolding',1) 
 
         # left truncation
-        self.truncate_left = getattr(extra, 'truncate_left', 0)
+        self.truncate_left = _get(extra, 'truncate_left', 0)
 
         # Term autoexpansion
-        self.autoexpand = getattr(extra, 'autoexpand', 0)
-        self.autoexpand_limit = getattr(extra, 'autoexpand_limit', 4)
+        self.autoexpand = _get(extra, 'autoexpand', 0)
+        self.autoexpand_limit = _get(extra, 'autoexpand_limit', 4)
 
         # maximum number of hits
-        self.numhits = getattr(extra, 'numhits', 999999999)
-
-        # name of stemmer or None
-        self.use_stemmer    = getattr(extra,'use_stemmer',    None) or None
+        self.numhits = _get(extra, 'numhits', 999999999)
 
         # default maximum distance for words with near search
-        self.near_distance  = getattr(extra,'near_distance',  5)
+        self.near_distance = _get(extra,'near_distance', 5)
 
         # Stopwords: either filename or StopWord object
-        self.use_stopwords     = getattr(extra,'use_stopwords',    None) or None
+        self.use_stopwords = _get(extra, 'use_stopwords', None) or None
         if self.use_stopwords:
             verifyClass(StopwordsInterface, self.use_stopwords.__class__)
      
         # Normalizer
-        self.use_normalizer = getattr(extra,'use_normalizer', None) or None
+        self.use_normalizer = _get(extra,'use_normalizer', None) or None
 
         # use converters from the ConvertersRegistry
-        self.use_converters = getattr(extra,'use_converters',0) 
+        self.use_converters = _get(extra,'use_converters',0) 
+
+        # Storage to be used
+        self.use_storage = _get(extra,'use_storage', 'StandardStorage') 
 
         # encoding
-        self.default_encoding = getattr(extra,'default_encoding', 'iso-8859-15') 
+        self.default_encoding = _get(extra,'default_encoding', 'iso-8859-15') 
 
         # check Parser
-        self.use_parser = getattr(extra, 'use_parser','PyQueryParser')
+        self.use_parser = _get(extra, 'use_parser','PyQueryParser')
+        
+        # Thesaurus
+        self.use_thesaurus = _get(extra, 'use_thesaurus', None)
+        self.thesaurus_mode = _get(extra, 'thesaurus_mode', None)
 
-        self.use_storage = 'StandardStorage'
         self.use_lexicon = 'StandardLexicon'
         self.clear()
 
 
     def clear(self):
-
         self._storage = StorageRegistry.get(self.use_storage)() 
         self._lexicon = LexiconRegistry.get(self.use_lexicon)(truncate_left=self.truncate_left)
 
     def getId(self):   return self.id
     def __len__(self): return len(self._storage)
     def __nonzero__(self): return not not self._unindex
-    def printIndex(self): self._storage.printStorage()
-    def availableStemmers(self): return Stemmer.availableStemmers()    
     def getLexicon(self): return self._lexicon
     def getStorage(self): return self._storage
 
     def index_object(self, documentId, obj, threshold=None):
         """ wrapper to handle indexing of multiple attributes """
-
         # needed for backward compatibility
         try: fields = self._indexed_fields
         except: fields  = [ self.id ]
 
         res = 0
-        wids = IISet()
-
+        all_wids = []
         for attr in fields:
             try:
-                result = self._index_object(documentId, obj, threshold, attr)
-                if result is not None:
-                    wids = union(wids, result)
+                wids = self._index_object(documentId, obj, threshold, attr)
+                if wids is not None:
+                    all_wids.extend(wids)
             except:
                 pass
 
@@ -230,17 +230,22 @@ class TextIndexNG(SimpleItem):
         except KeyError:
             o_wids = IISet()
 
-        remove_wids = difference(o_wids, wids)
-        insert_wids = difference(wids, o_wids)
+        all_wids_set = IISet(all_wids)
+        remove_wids = difference(o_wids, all_wids_set)
+        insert_wids = difference(all_wids_set, o_wids)
+        insert_dict = {}   # hash wids to dict for performance reasons
+        for wid in insert_wids.keys(): insert_dict[wid] = 1
 
         if len(remove_wids) > 0:
             self._storage.removeWordIdsForDocId(documentId, remove_wids) 
-        self._storage.insert(wids, documentId)
-        return len(wids)
+        if all_wids:
+            self._storage.insert([w for w in all_wids if insert_dict.has_key(w)], documentId)
+        return len(all_wids)
         
     def _index_object(self, documentId, obj, threshold=None, attr=''):
-        
-        source = mimetype = encoding = None
+
+        encoding = self.default_encoding
+        source = mimetype = None
 
         # This is to support foreign file formats that
         # are stored as "File" objects when searching
@@ -254,7 +259,12 @@ class TextIndexNG(SimpleItem):
 
         elif obj.meta_type in ('File', 'Portal File') and  \
            attr in ('PrincipiaSearchSource', 'SearchableText'):
-            source = str(obj)
+
+            source= getattr(obj, attr, None)
+            if source and not self.use_converters:
+                if callable(source): source = source()
+            else:              
+                source = str(obj)
             mimetype = obj.content_type
 
         elif obj.meta_type == 'ExtFile' and \
@@ -287,7 +297,7 @@ class TextIndexNG(SimpleItem):
             try:
                 source = getattr(obj, attr)
                 if callable(source): source = source()
-                if not isinstance(source, UnicodeType):
+                if not isinstance(source, unicode):
                     source = str(source)
             except (AttributeError, TypeError):
                 return None
@@ -297,13 +307,15 @@ class TextIndexNG(SimpleItem):
         # of the data.
 
         if self.use_converters:
-            if mimetype is None:
+            if mimetype is None or mimetype == 'application/octet-stream':
                 mimetype, encoding = guess_content_type(obj.getId(), source)
+                if not encoding:
+                    encoding = self.default_encoding
 
             try: 
                 converter = ConverterRegistry.get(mimetype)
             except RegistryException: 
-                LOG1('textindexng', ERROR, '%s could not be converted because no converter could be found for %s' % (obj.absolute_url(1), mimetype))
+                LOG('textindexng', ERROR, '%s could not be converted because no converter could be found for %s' % (obj.absolute_url(1), mimetype))
                 return None
 
             if converter:
@@ -313,7 +325,7 @@ class TextIndexNG(SimpleItem):
                     try:
                         source = converter.convert(source)
                     except:
-                        LOG1('textindexng', ERROR, '%s could not be converted' % obj.absolute_url(1), error=sys.exc_info())
+                        LOG('textindexng', ERROR, '%s could not be converted' % obj.absolute_url(1), error=sys.exc_info())
                         return None
 
             if obj.meta_type == 'Portal File': 
@@ -332,7 +344,7 @@ class TextIndexNG(SimpleItem):
         # Python's default encoding.
         # After getting the encoding, we convert the data to unicode.
 
-        if isinstance(source, StringType):
+        if isinstance(source, str):
             if encoding is None:
                 try: encoding = self.default_encoding
                 except: encoding = self.default_encoding = 'iso-8859-15'
@@ -345,12 +357,15 @@ class TextIndexNG(SimpleItem):
             try:
                 source = unicode(source, encoding, 'strict')
             except UnicodeDecodeError:
-                LOG1('textindexng', WARNING, 'UnicodeDecodeError raised from %s - ignoring unknown unicode characters'  % obj.absolute_url(1))
+                LOG('textindexng', WARNING, 'UnicodeDecodeError raised from %s - ignoring unknown unicode characters'  % obj.absolute_url(1))
                 source = unicode(source, encoding, 'ignore')
  
-        elif isinstance(source, UnicodeType):  pass
+        elif isinstance(source, unicode):  pass
         else: raise TXNGError,"unknown object type" 
 
+        source = source.strip()
+        if not source: return None
+
         # Normalization: apply translation table to data
         if self.use_normalizer:
             source = NormalizerRegistry.get(self.use_normalizer).process(source)    
@@ -363,20 +378,16 @@ class TextIndexNG(SimpleItem):
                    separator    = self.splitter_separators,
                    maxlen       = self.splitter_max_len,
                    singlechar   = self.splitter_single_chars
-                   ).split(_source, encoding)
+                   ).split(_source)
 
         #  remove stopwords from data
         if self.use_stopwords:
             words = self.use_stopwords.process( words ) 
 
-        # Stem all words in one run
-        if self.use_stemmer:
-            words = Stemmer.Stemmer( self.use_stemmer ).stem(words)
-
         # We pass the list of words to the corresponding lexicon
         # and obtain a list of wordIds. The "old" TextIndex iterated
         # over every single words (overhead).
-        return IISet(self._lexicon.getWordIdList(words))
+        return self._lexicon.getWordIdList(words)
 
     def unindex_object(self, documentId): 
         """ carefully unindex document with Id 'documentId'
@@ -416,23 +427,25 @@ class TextIndexNG(SimpleItem):
  
         query = record.keys[0]
         encoding = record.get('encoding', self.default_encoding)
-        if isinstance(query, StringType): query = unicode(query, encoding)
+        if isinstance(query, str): query = unicode(query, encoding)
         P = ParserRegistry.get( query_parser )
         parsed_query = P(query.strip(), operator=query_operator)
         if not parsed_query:
             raise TXNGError,"Bad query: '%s'" % q
 
-        LOG.debug('parsed query: %s' % parsed_query)
-
         evaluator = Evaluator(self)
         evaluator.autoexpand = record.get('autoexpand', self.autoexpand)
         evaluator.near_distance = record.get('near_distance', self.near_distance)
 
         numhits = record.get('numhits', self.numhits)
         resultset = evaluator(parsed_query)
-        resultset.cosine_ranking(self, numhits)
 
-        return  resultset.result(), (self.id,) 
+        if self.getStorage().providesWordFrequencies():
+            resultset.cosine_ranking(self, numhits)
+            return  resultset.result(), (self.id,) 
+        else:
+            return  resultset.docIds(), (self.id,) 
+            
 
 
     ################################################################
@@ -451,6 +464,12 @@ class TextIndexNG(SimpleItem):
         if self.use_stopwords:
             words = self.use_stopwords.process( words ) 
 
+        if self.use_thesaurus and self.thesaurus_mode == 'expand_always':
+            TH = ThesaurusRegistry.get(self.use_thesaurus)
+            for word in words[:]:
+                r = TH.getTermsFor(word)
+                words.extend(r)
+
         for word in words:
 
             # perform casefolding if necessary
@@ -460,10 +479,6 @@ class TextIndexNG(SimpleItem):
             if self.use_normalizer:
                 word = NormalizerRegistry.get(self.use_normalizer).process(word)    
  
-            # Stem the word if necessary        
-            if self.use_stemmer:
-                word  = Stemmer.Stemmer(self.use_stemmer).stem(word)
-
             used_words[word] = 1.0
 
             wid = self._lexicon.getWordId(word)
@@ -495,10 +510,6 @@ class TextIndexNG(SimpleItem):
     def lookupWordsByPattern(self,word):
         """ perform full pattern matching """
 
-        if self.use_stemmer:
-            raise TXNGError, \
-                "pattern matching not allowed with enabled stemming"
-
         if self.splitter_casefolding: word = word.lower()
         words = self._lexicon.getWordsForPattern(word)
 
@@ -510,10 +521,6 @@ class TextIndexNG(SimpleItem):
         if self.use_normalizer:
             word = NormalizerRegistry.get(self.use_normalizer).process(word)    
 
-        if self.use_stemmer:
-            raise TXNGError, \
-                "right truncation not allowed with enabled stemming"
-
         if self.splitter_casefolding: word = word.lower()
         if right:
             words = self._lexicon.getWordsForRightTruncation(word)
@@ -657,10 +664,6 @@ class TextIndexNG(SimpleItem):
         except:
             return [ self.id ]
 
-    def getDebugMode(self):
-        """ return debug mode """
-        return LOG.status()
-
 
     ###################################################################
     # Stopword handling
@@ -741,6 +744,21 @@ class TextIndexNG(SimpleItem):
         paths.sort()
         return paths
 
+    ###################################################################
+    # Cleanup vocabulary
+    ###################################################################
+
+    def manage_cleanVocabulary(self):
+        """ cleanup the vocabulary """
+
+        wids = list(self._lexicon.getWordIds())
+        for wid in wids:
+            docids = self._storage.getDocumentIdsForWordId(wid)
+            if not docids:
+                self._lexicon.removeWordId(wid)
+
+        return 'Vocabulary cleaned'
+
     ###################################################################
     # TextIndexNG preferences 
     ###################################################################
@@ -749,9 +767,6 @@ class TextIndexNG(SimpleItem):
                                REQUEST=None,RESPONSE=None,URL2=None):
         """ preferences of TextIndex """
 
-        if debug_mode: LOG.debug_on()
-        else: LOG.debug_off()
-
         for x in ('near_distance', ): 
 
             if hasattr(extra,x):
@@ -764,6 +779,33 @@ class TextIndexNG(SimpleItem):
             RESPONSE.redirect(URL2 + 
                 '/manage_main?manage_tabs_message=Preferences%20saved')
 
+    def manage_checkIndex1(self):
+        """ check index (only for internal tests) """
+      
+        # check lexikon
+        fwidx = self._lexicon._forward_idx
+        revidx = self._lexicon._inverse_idx
+        all_wids = fwidx.values()
+        assert len(fwidx) == len(revidx) 
+        for word, wid in fwidx.items():
+            assert revidx[wid] == word
+
+        # check storage
+        fwidx = self._storage._forward_idx
+        revidx = self._storage._reverse_idx
+
+        all_docids = revidx.keys()
+        for wid,docids in fwidx.items():
+            assert wid in all_wids
+            for docid in (isinstance(docids, int) and [docids] or docids.keys()):
+                assert docid in all_docids
+
+        for docid in revidx.keys():
+            for wid in self._storage.getWordIdsForDocId(docid):
+                assert wid in all_wids
+        return 'Index seems to be consistent'
+
+
     manage_workspace  = DTMLFile("dtml/manageTextIndexNG",globals())
     manage_stopwords  = DTMLFile("dtml/manageStopWords",globals())
     manage_normalizer = DTMLFile("dtml/manageNormalizer",globals())
