@@ -10,7 +10,7 @@
 """
 WinWord converter
 
-$Id: doc.py,v 1.12 2004/03/16 06:05:27 ajung Exp $
+$Id: doc.py,v 1.13 2004/10/18 16:00:00 ajung Exp $
 """
 
 import os, sys
@@ -31,9 +31,9 @@ class Converter(BaseConverter):
         
         tmp_name = self.saveFile(doc)
         if sys.platform == 'win32':
-            return self.execute('wvWare --nographics -x "%s" "%s" 2> nul:' % (wvConf_file, tmp_name))
+            return self.execute('wvWare -c utf-8 --nographics -x "%s" "%s" 2> nul:' % (wvConf_file, tmp_name))
         else:
-            return self.execute('wvWare --nographics -x "%s" "%s" 2> /dev/null' % (wvConf_file, tmp_name))
+            return self.execute('wvWare -c utf-8 --nographics -x "%s" "%s" 2> /dev/null' % (wvConf_file, tmp_name))
 
     def convert2(self, doc, encoding, mimetype):
-        return self.convert(doc), 'iso-8859-15'
+        return self.convert(doc), 'utf-8'
