@@ -10,13 +10,14 @@
 """
 a stupid HTML to Ascii converter
 
-$Id: html.py,v 1.11 2004/03/16 06:05:27 ajung Exp $
+$Id: html.py,v 1.17 2004/09/08 20:20:12 ajung Exp $
 """
 
-from types import StringType
 from Products.TextIndexNG2.BaseConverter import BaseConverter
+from entities import convert_entities
 from stripogram import html2text
 
+
 class Converter(BaseConverter):
 
     content_type = ('text/html',)
@@ -29,6 +30,12 @@ class Converter(BaseConverter):
                          ignore_tags=('img',),
                          indent_width=0,
                          page_width=256)
-        
+         
     def convert2(self, doc, encoding, mimetype):
-        return self.convert(doc), 'iso-8859-15'
+        # convert to unicode
+        if not isinstance(doc, unicode):
+            doc = unicode(doc, encoding, 'replace')
+        doc = convert_entities(doc)
+        result = self.convert(doc)
+        # convert back to utf-8
+        return result.encode('utf-8'), 'utf-8'
