@@ -10,7 +10,7 @@
 """
 a stupid null converter
 
-$Id: null.py,v 1.8 2004/03/16 06:05:27 ajung Exp $
+$Id: null.py,v 1.9 2005/02/20 12:13:31 ctheune Exp $
 """
 
 from Products.TextIndexNG2.BaseConverter import BaseConverter
@@ -24,4 +24,4 @@ class Converter(BaseConverter):
         return doc
 
     def convert2(self, doc, encoding, mimetype):
-        return self.convert(doc), 'iso-8859-15'
+        return self.convert(doc), encoding
