@@ -10,11 +10,10 @@
 """
 PowerPoint converter
 
-$Id: ppt.py,v 1.12 2004/03/16 06:05:27 ajung Exp $
+$Id: ppt.py,v 1.13 2004/09/08 20:20:12 ajung Exp $
 """
 
-import os, sys
-from Globals import package_home
+import sys
 from Products.TextIndexNG2.BaseConverter import BaseConverter
 from stripogram import html2text
 
