@@ -10,12 +10,15 @@
 """
 A stupid HTML to Ascii converter
 
-$Id: sgml.py,v 1.5 2004/03/16 10:51:35 ajung Exp $
+$Id: sgml.py,v 1.8 2004/07/03 10:37:57 ajung Exp $
 """
 
-from types import StringType
+import re
 from Products.TextIndexNG2.BaseConverter import BaseConverter
 from StripTagParser import StripTagParser
+from entities import convert_entities
+
+encoding_reg = re.compile('encoding="(.*?)"')
 
 class Converter(BaseConverter):
 
@@ -31,4 +34,14 @@ class Converter(BaseConverter):
         return str(p)
 
     def convert2(self, doc, encoding, mimetype):
-        return self.convert(doc), 'iso-8859-15'
+
+        # Use encoding from XML preamble if present
+        mo = encoding_reg.search(doc)
+        if mo:
+            encoding = mo.group(1)
+        
+        if not isinstance(doc, unicode):
+            doc = unicode(doc, encoding, 'replace')
+        doc = convert_entities(doc)
+        doc = doc.encode('utf-8')
+        return self.convert(doc), 'utf-8'
