@@ -10,19 +10,17 @@
 """
 Excel Converter
 
-$Id: xls.py,v 1.4 2004/03/16 06:05:27 ajung Exp $
+$Id: xls.py,v 1.6 2004/09/08 20:20:12 ajung Exp $
 """
 
-import os, sys
-
-from Globals import package_home
+import sys
 from Products.TextIndexNG2.BaseConverter import BaseConverter
 
 class Converter(BaseConverter):
 
     content_type = ('application/msexcel','application/ms-excel','application/vnd.ms-excel')
     content_description = "Microsoft Excel"
-    depends_on = 'xls2cvs'
+    depends_on = 'xls2csv'
 
     def convert(self, doc):
         """Convert Excel document to raw text"""
