@@ -45,3 +45,9 @@ class LexiconInterface(Interface.Base):
 
     def getWordsForSubstring(sub):
         """ return all words that match the given substring """
+
+    def getWordIds():
+        """ return all wids """
+
+    def removeWordId(wid):
+        """ remove word id 'wid' """
