@@ -36,6 +36,8 @@ class StorageInterface(Interface.Base):
         """ return a sequence of words contained in the document with 
             ID 'docId'
         """
-
     def getWordFrequency(docId, wid):
         """ return the # occurences of a wid with a document """   
+
+    def providesWordFrequencies():
+        """" returns True|False if it support storing word frequencies """
