@@ -10,22 +10,25 @@
 
 # check for parser modules
 
+import os
 from Products.TextIndexNG2.Registry import NormalizerRegistry
 from Products.TextIndexNG2.Normalizer import FileNormalizer
+from Products.TextIndexNG2 import fast_startup
 
-import os
-_oldcwd = os.getcwd()
 
-os.chdir(__path__[0])
-files = os.listdir('.')
-files = [ x   for x in files if x.endswith('.txt') ] 
+if not fast_startup:
 
-for f  in files:
+    _oldcwd = os.getcwd()
 
-    n = FileNormalizer(f)
-    try: NormalizerRegistry.register(n.getLanguage(), n) 
-    except: 
-        pass
+    os.chdir(__path__[0])
+    files = os.listdir('.')
+    files = [ x   for x in files if x.endswith('.txt') ] 
 
-os.chdir(_oldcwd)
+    for f  in files:
 
+        n = FileNormalizer(f)
+        try: NormalizerRegistry.register(n.getLanguage(), n) 
+        except: 
+            pass
+
+    os.chdir(_oldcwd)
