@@ -7,25 +7,14 @@
 #
 ###########################################################################
 
-
-# check for parser modules
-
 from Products.TextIndexNG2.Registry import ParserRegistry
 
-import os
-_oldcwd = os.getcwd()
-
-os.chdir(__path__[0])
-parsers = os.listdir('.')
-parsers = [ p   for p in parsers if os.path.isdir(p) and not p in ['CVS'] ]
-
-for p  in parsers:
-
-    mod = __import__(p, globals(), globals(), __path__)
-
-    try: ParserRegistry.register( p, mod.Parser() )
-    except: pass
-
-del parsers
-os.chdir(_oldcwd)
+import PyQueryParser
+import DumbQueryParser
+import GermanQueryParser
+import FrenchQueryParser
 
+ParserRegistry.register('PyQueryParser', PyQueryParser.Parser())
+ParserRegistry.register('DumbQueryParser', DumbQueryParser.Parser())
+ParserRegistry.register('GermanQueryParser', GermanQueryParser.Parser())
+ParserRegistry.register('FrenchQueryParser', FrenchQueryParser.Parser())
